package ladasha.io;

import java.io.File;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.Sequence;
import javax.sound.midi.Sequencer;


public class MidiFileHandler {	
  private File file = null;
  private Sequencer seq1 = null;
  private Sequence seq = null;
  private static MidiFileHandler instance = null;
  private static Logger logger = Logger.getInstance();

  private MidiFileHandler() {
    new Thread(new Runnable() {
      public void run() {	
        try {
          seq1 = MidiSystem.getSequencer();
          if(seq1 == null) return; // throw exception here in the future
          else seq1.open();
        } catch (Exception ex) { logger.log(ex);}
      }
    }).start();
  }
  
  public static MidiFileHandler getInstance() {
    logger.log("obtaining instance of MidiFileHandler");
    if(instance == null) instance = new MidiFileHandler();
    return instance;
  }

  public boolean loadMidiFile(File cake) {
    logger.log("MidiFileHandler: loadMidiFile(cake): loading MIDI file");
    this.file = cake;
    if(file == null) return false; // throw exception here in the future
    try{        
      // sequencer gets file
      seq =  MidiSystem.getSequence(cake);
      seq1.setSequence(seq);
    } catch(Exception ex) {
      logger.log(ex);
    }
    return true;
  } // end method
  
  public Sequence getMidiSequence() {
    logger.log("MidiFileHandler: getMidiSequence(): getting MIDI sequence");
    if(seq == null)
      return null; // throw exception here in the future
    else
      return seq;
  }
 
	public Sequencer getMidiSequencer() {
		logger.log("MidiFileHandler: getMidiSequencer(): getting MIDI sequencer");
		if(seq1 == null) return null;
		else return seq1;
	}

  public int getTracks() {
    logger.log("MidiFileHandler: getTracks(): getting number of tracks");
    return seq.getTracks().length;
  }

  public void stop() {
    logger.log("MidiFileHandler: stop(): stopping MIDI sequence");
    if(seq1 == null) return; // throw exception here in the future
    try {
      seq1.stop();
    } catch (Exception ex) { logger.log(ex); }
  }

  public boolean isPlaying() {
    return seq1.isRunning();
  }

  public void preview(int millis) {
    logger.log("MidiFileHandler: preview(millis): previewing for " + millis + " milliseconds");
    if(seq1 == null) return; // throw exception here in the future
    try{
      Thread t = new Thread(new Runnable() {
      	public void run() { 
      	  seq1.start();  
        }
      });
      t.start();
      // lets the song get previewed for (millis/1000) sec
      Thread.sleep(millis);
      seq1.stop();
    } catch(Exception ex){
      logger.log(ex);
    }
  }
  
  public void preview() {
    preview(10000);
  }
  
  public void previewUntilStop() {
    logger.log("MidiFileHandler: previewUntilStop(): previewing MIDI sequence");
    if(seq1 == null) return; // throw exception here in the future
    try{
      Thread t = new Thread(new Runnable() {
      	public void run() { 
      	  seq1.start();
        }
      });
      t.start();
      } catch(Exception ex){
      ex.printStackTrace();
    }
  }
}// end class
